'''Functions to get information about a Debian-style package archive.'''

import apt_pkg, urllib, gzip, tempfile, os

def package_list_map(url):
    '''Read a package list from the given URL and parse it into a map 
    package->key->value.

    E. g. package_list_map('Sources.gz')['dpkg']['Version'] would deliver the
    Version of the dpkg source package.'''

    # write the uncompressed downloaded list into a temporary file
    downloadfile = urllib.urlretrieve(url, tempfile.mkstemp()[1])[0]
    list = tempfile.TemporaryFile()
    try:
        print >> list, gzip.open(downloadfile).read()
    except IOError:
        print >> list, open(downloadfile).read()
    os.unlink(downloadfile)
    list.seek(0)

    map = {}
    parser = apt_pkg.ParseTagFile(list)
    while parser.Step() == True:
        pkg = {}
        for key in parser.Section.keys():
            if key != 'Package':
                pkg[key] = parser.Section.get(key)
        map[parser.Section.get('Package')] = pkg
    return map

# tests 

if __name__ == '__main__':
    map = package_list_map('/var/lib/apt/lists/archive.ubuntu.com_ubuntu_dists_breezy_main_source_Sources')
    print map['dpkg']
    print map['pmount']['Maintainer']

    map = package_list_map('http://archive.ubuntu.com/ubuntu/dists/breezy/main/source/Sources.gz')
    print map['dpkg']
    print map['pmount']['Maintainer']
