'''pkg_classify.py: classify Ubuntu source packages into different sections
like "gnome", "kde", and "" (which means "all the rest").'''

import archive_info

class PackageClassificator:
    def __init__(self, release, domain_map, archive_root = 'http://archive.ubuntu.com/ubuntu'):
        # read domain_map.txt
        self.domain_package = {}
        for line in domain_map.splitlines():
            f = line.split()
            if len(f) != 2:
                continue
            self.domain_package[f[1]] = f[0]

        release = release.split('-')[0]

        # get archive map
        self.package_info = archive_info.package_list_map(
            '%s/dists/%s/main/source/Sources.gz' % (archive_root, release))

    def classify_domain(self, domain):
        '''Classify given domain.

        Raises an exception if the domain is unknown.'''

        try:
            pkg = self.domain_package[domain]
        except KeyError:
            raise Exception, 'Unknown domain %s' % domain
        return self.classify_package(pkg)

    def classify_package(self, package):
        '''Classify given source package.

        Raises an exception if the package is unknown.'''

        i = self.package_info[package]
        desc = (package + i['Binary'] + i['Section'] + i['Maintainer'] + 
            i.get('Build-Depends', '')).lower()
        has_gnome = (desc.find('gnome') >= 0) or package.startswith('gtk')
        has_kde = (desc.find('kde') >= 0)
        if has_gnome and not has_kde:
            return 'gnome'
        elif has_kde and not has_gnome:
            return 'kde'
        else:
            return ''

# test

if __name__ == '__main__':
    testmap = """
gnome-session gnome-session-2.0
gtk+1.2 gtk+
gtkspell gtkspell
gaim gaim
knetworkconf knetworkconfmodule
postgresql-8.0 pg_dump-8.0
"""
    pc = PackageClassificator('breezy', testmap)
    print 'domain gnome-session-2.0 (expected: gnome):', pc.classify_domain('gnome-session-2.0')
    print 'package gnome-session (expected: gnome):', pc.classify_package('gnome-session')
    print 'domain gtk+ (expected: gnome):', pc.classify_domain('gtk+')
    print 'domain gtkspell (expected: gnome):', pc.classify_domain('gtkspell')
    print 'domain gaim (expected: empty):', pc.classify_domain('gaim')
    print 'domain knetworkconfmodule (expected: kde):', pc.classify_domain('knetworkconfmodule')
    print 'package knetworkconf (expected: kde):', pc.classify_package('knetworkconf')
    print 'package meta-kde (expected: kde):', pc.classify_package('meta-kde')
    print 'domain pg_dump-8.0 (expected: empty):', pc.classify_domain('pg_dump-8.0')
    print 'package postgresql-8.0 (expected: empty):', pc.classify_package('postgresql-8.0')

    pc = PackageClassificator('breezy-updates', testmap)
    print 'domain gnome-session-2.0 (expected: gnome):', pc.classify_domain('gnome-session-2.0')
    print 'package gnome-session (expected: gnome):', pc.classify_package('gnome-session')
    print 'domain gtk+ (expected: gnome):', pc.classify_domain('gtk+')
    print 'domain gtkspell (expected: gnome):', pc.classify_domain('gtkspell')
    print 'domain gaim (expected: empty):', pc.classify_domain('gaim')
    print 'domain knetworkconfmodule (expected: kde):', pc.classify_domain('knetworkconfmodule')
    print 'package knetworkconf (expected: kde):', pc.classify_package('knetworkconf')
    print 'package meta-kde (expected: kde):', pc.classify_package('meta-kde')
    print 'domain pg_dump-8.0 (expected: empty):', pc.classify_domain('pg_dump-8.0')
    print 'package postgresql-8.0 (expected: empty):', pc.classify_package('postgresql-8.0')
